package lsedit;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.Cursor;
import java.awt.Toolkit;
import java.awt.FontMetrics;

public class MatrixLayout extends LandscapeLayouter  implements ToolBarEventHandler {

  protected final static int BORDER = 30;	// 1/30 of graph all round
  protected final static int GAP    = 5;	// Make gap 1/5 of space

  public MatrixLayout(LandscapeEditorCore ls) 
  {
	super(ls);
  }

/****************************************************************************/

  public String getMenuLabel() 
  {
	return "Layout in a matrix";
  } 

/***************************************************************************/


	// The doLayout method executes the Matrix algorithm
	// Assumption: All boxes selected are in the same container.

	public void doLayout1(Vector selectedBoxes, EntityInstance container, boolean update) 
	{
		int				size = selectedBoxes.size();
		int				width, height, dimension, x, y, w, h, gapw, gaph, w1, h1, x1, y1, row, col, i;
		EntityInstance	e;
		double			xrel, yrel, widthrel, heightrel;

		width  = container.getWidth();
		height = container.getHeight();
		x      = width  / BORDER;
		y      = height / BORDER;
		w      = width  - 2 * x;
		h      = height - 2 * y;

		if (w < 1 || h < 1) {
			return;
		}

		switch (size) {
		case 0:
			return;
		case 1:
			dimension = 1;
			gapw      = 0;
			gaph      = 0;
			w1        = w;
			h1        = h;
			break;
		default:
			dimension = (int) (Math.ceil(Math.sqrt(size)));
			w1        = w / dimension;
			h1        = h / dimension;
			gapw      = w1 / (GAP*2); 
			gaph      = h1 / (GAP*2);
		}
		
		if (w1 < 1 || h1 < 1) {
			return;
		}

		widthrel  = ((double) (w1 - gapw))/((double) width);
		heightrel = ((double) (h1 - gaph))/((double) height);

		SortVector.byString(selectedBoxes, true /* ascending */);

		row = col = 0;
		for (i = 0; i < size; ++i) {
			e = (EntityInstance) selectedBoxes.elementAt(i);
			x1 = x;
			if (col != 0) {
				x1 += col*w1 + gapw;
			}
			xrel = ((double) x1)/((double) width);
			y1 = y;
			if (row != 0) {
				y1 += row*h1 + gaph;
			}
			yrel = ((double) y1)/((double) height);

			if (update) {
				e.updateRelLocal(xrel, yrel, widthrel, heightrel);
			} else {
				e.setRelLocal(xrel, yrel, widthrel, heightrel);
			}
			if (++col == dimension) {
				++row;
				col = 0;
		}	}

//		System.out.println("Matrix done");

	} // doLayout


  // The doLayout method executes the Coffman-Graham Layer Assignment
  // algorithm and the Sugiyama algorithm on the boxes selected.
  // Assumption: All boxes selected are in the same container.

	public String doLayout(Diagram dg) 
	{
		EntityInstance parent;

		// get user's selection of boxes to be laid out

		m_ls.setLayouter(this);
		m_ls.setCursor(Cursor.WAIT_CURSOR);

		Vector selectedBoxes = dg.getGroup();
		if (selectedBoxes == null) {
			  beep();
			  return "No group selected";
		}

		parent = parentOfSet(selectedBoxes);
		if (parent == null) {
			return	"Matrix layouter requires that all things laid out share same parent";
		}
		dg.beginUndoRedo("Matrix layout");
		doLayout1(selectedBoxes, parent, true /* Updating */);
		dg.endUndoRedo();

		m_ls.setCursor(Cursor.DEFAULT_CURSOR);

		return "Graph redrawn using Matrix Layout";
	} // doLayout

	public void processKeyEvent(int key, int modifiers, Object object) 
	{
		Diagram	dg;
		String	rmsg;

		dg = m_ls.getDiagram();
		if (dg != null) {
			rmsg = doLayout(dg);
			m_ls.doFeedback(rmsg);
			dg.rescaleDiagram();
			m_ls.repaintDg();
	}	}
} 





